@@ -618,8 +618,12 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 				mstruct->setApproximate(true, true);
 			}
 			if(!mexp.isOne()) mstruct->raise(mexp);
-			if(mvalue.isOne()) mvalue.set_nocopy(*mstruct);
-			else mvalue.multiply_nocopy(mstruct, true);
+			if(mvalue.isOne()) {
+				mvalue.set_nocopy(*mstruct);
+				mstruct->unref();
+			} else {
+				mvalue.multiply_nocopy(mstruct, true);
+			}
 		}
 	}
 	return mvalue;
@@ -710,8 +714,12 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 			mstruct->setApproximate(true, true);
 		}
 		if(!mexp.isOne()) mstruct->raise(mexp);
-		if(mvalue.isOne()) mvalue.set_nocopy(*mstruct);
-		else mvalue.multiply_nocopy(mstruct, true);
+		if(mvalue.isOne()) {
+			mvalue.set_nocopy(*mstruct);
+			mstruct->unref();
+		} else {
+			mvalue.multiply_nocopy(mstruct, true);
+		}
 	}
 	if(i_exp != 1) mexp.multiply(i_exp);
 	return mvalue;
